/**
 * Simple External Command Tuner Plugin for SageTV
 * 
 * 
 */

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "ExtTunerPlugin.h"

#ifndef DEVICE_NAME
#define DEVICE_NAME "External Command Tuner"
#endif

#ifndef LOG_FILE
#define LOG_FILE "ext-command-tuner.log"
#endif

#ifndef CONFIG_FILE
#define CONFIG_FILE "ext-command-tuner-remotes.cfg"
#endif

/**
 * Simply non-efficient logger.
 */
void _log(char *pref, char *fmt, ...) {
	va_list ap;
	FILE *f1;
	f1 = fopen(LOG_FILE, "a");
	va_start(ap, fmt);
	fprintf(f1, "%s: ", pref);
	vfprintf(f1, fmt, ap);
	fprintf(f1,"\n");
	va_end(ap);
	fflush(f1);
	fclose(f1);
}

#define INFO(s) _log("INFO", s)
#define INFO1(f,a) _log("INFO", f,a)
#define INFO2(f,a,b) _log("INFO", f,a,b)
#define INFO3(f,a,b,c) _log("INFO", f,a,b,c)
#define ERROR(s) _log("ERROR", s)
#define ERROR1(f,a) _log("ERROR", f,a)
#define ERROR2(f,a,b) _log("ERROR", f,a,b)

/**
 * simple method to allocate and copy string
 */
char *alloc_string(char *str) {
	if (str==NULL) return NULL;
	char *newStr = (char *)malloc(strlen(str)+1); 
	strcpy(newStr, str);
	return newStr;
}



command* CreateCommand(unsigned char *Name) {
	struct command *Com; //pointer to new command structure

	Com = (struct command*)malloc(sizeof(struct command));//allocate space for a command structure
	if (Com == NULL) {
		return Com;
	}
	Com->name = Name; //copy values
	Com->next = NULL;
	Com->pattern = NULL;
	return Com; //return pointer to new command structure
}

int NeedBitrate() {
	INFO("NeedBitrate");
	return 0;
}
int NeedCarrierFrequency() {
	INFO("NeedCarrierFrequency");
	return 0;
}

const char* DeviceName() {
	INFO("DeviceName");
	return DEVICE_NAME;
}

int OpenDevice(int ComPort) {
	INFO("OpenDevice");
	return 1;
}

void CloseDevice(int devHandle) {
	INFO("CloseDevice");
}

unsigned long FindBitRate(int devHandle) {
	INFO("FindBitRate");
	return 0;
}

unsigned long FindCarrierFrequency(int devHandle) {
	INFO("FindCarrierFrequency");
	return 0;
}

void AddRemote(struct remote *Remote, struct remote **head) {
	struct remote *Temp; //Local remote structure
        INFO1("AddRemote(%s)", Remote->name);
	if (!(*head)) { //if there are no structures in the list
		*head = Remote; //then assign this one to head.
		(*head)->next = NULL;
	} else //otherwise, add to end of list
	{
		Temp = *head;
		while (Temp->next) {
			Temp = Temp->next; //find the last structure in list
		}
		Temp->next=Remote; //assign the next field to the new structure
		Temp->next->next = NULL; //assign the next field of the new structure to NULL
	}
}

void AddCommand(struct command *Command, struct command **Command_List) {
	struct command *Temp; //temporary command structure pointer
	INFO1("AddCommand(%s)", Command->name);
	if (!(*Command_List)) { //if no commands in list, assign Command_List
		(*Command_List) = Command; //to the command structure
		(*Command_List)->next = NULL;
	} else {
		Temp = (*Command_List); //ELSE add to end of list of commands
		while (Temp->next) {
			Temp = Temp->next;
		}
		Temp->next=Command;
		Temp->next->next = NULL;
	}
}

static char loadedDevName[256];
remote* LoadRemotes(const char* pszPathName) {
	char buf[1024];
	
	if (pszPathName==NULL) {
		INFO("LoadRemotes Called");
	} else {
		INFO1("LoadRemoted(%s)", pszPathName);
	}

	
	
	remote *head= NULL;
	INFO1("Loading Remotes from %s", CONFIG_FILE);
	
	FILE *f1 = fopen(CONFIG_FILE, "r");
	if (f1==NULL) {
		ERROR1("Failed to open remotes config file %s", CONFIG_FILE);
		return NULL;
	}
	
	char *name;
	char *cmd;
	remote *newRemote;
	command *newCmd;
	while (fgets(buf, 1024, f1)) {
		// ignore comments and empty lines
		if (strlen(buf)==0 || buf[0]==' ' || buf[0]=='#') continue;
		
		name=buf;
		cmd = strchr(buf, ',');
		*cmd = 0;
		cmd++;
		while (*cmd == ' ') cmd++;
		
		newRemote = CreateRemote(alloc_string(name));
		
		// store the command to execute in the remote structure's command list...
		// this is not what it's supposed to be used for... but that's the only
		// place I could store it without creating a separate storage area
		newCmd = CreateCommand(alloc_string(cmd));
		
		strcpy(loadedDevName, cmd);
		
		newRemote->command = newCmd;
		
		if (head==NULL) {
			head = newRemote;
		} else {
			AddRemote(newRemote, &head);
		}
	}
	
	fclose(f1);
	
	return head;
}

remote* CreateRemote(unsigned char *Name) {
	INFO1("Creating/Adding Remote: %s", Name);
	remote *Remote;

	Remote = (struct remote*)malloc(sizeof(struct remote)); //allocate space for a remote structure
	if (Remote == NULL) {
		return Remote;
	}
	Remote->name = Name; //copy values
	Remote->carrier_freq = 0;
	Remote->bit_time = 0;
	Remote->command = NULL;
	Remote->next = NULL;
	return Remote; //return pointer to remote structure

}

void InitDevice() {
	INFO("InitDevice Called");
}

command* RecordCommand(int devHandle, unsigned char *Name) {
	INFO("RecordCommand");
	return 0;
}

void PlayCommand(int devHandle, remote *remote, unsigned char *name, int tx_repeats) {
	INFO3("PlayCommand Called: Remote: %s; Command: %s; Repeats: %d", remote->name, name, tx_repeats);
    char cmd[1024];
    sprintf(cmd,remote->command->name, name);
    if (system(cmd)!=0) {
    	ERROR1("Failed to execute Tuner Command: %s", cmd);
    } else {
    	INFO1("Executed Tuner Command without Error: %s", cmd);
    }
}

void FreeRemotes(remote **head) {
	INFO("FreeRemotes Called.");
	command *Temp_Com; //temporary command pointer
	remote *Temp_Rem; //temporary remote pointer
	pattern *temp_pat; //temporary pattern pointer

	while (*head) {
		Temp_Rem = *head;
		Temp_Com = (*head)->command;
		while (Temp_Com) {
			(*head)->command = (*head)->command->next;
			temp_pat = Temp_Com->pattern;
			while (temp_pat) {
				Temp_Com->pattern = Temp_Com->pattern->next;
				free(temp_pat->bytes);
				free(temp_pat);
				temp_pat = Temp_Com->pattern;
			}
			free(Temp_Com->name); //free command list
			free(Temp_Com);
			Temp_Com = (*head)->command;
		}
		(*head) = (*head)->next;
		free(Temp_Rem->name); //free remote data
		free(Temp_Rem);
	}
	*head = NULL;
}

void DumpRemotes(remote *head) {
	while (head) {
		printf("RemoteName: %s\n", head->name);
		printf(" RemoteCmd: %s\n\n", head->command->name);
		head = head->next;
	}
}

int CanMacroTune(void) {
	INFO("CanMacroTune");
	return 1;
}
void MacroTune(int devHandle, int channel) {
	INFO2("MacroTune: Handle: %d,Channel %d",devHandle,channel);
	
	char cmd[1024];
	char    channelStr[10];  // if more than 10 digits, we have problems
	sprintf(channelStr, "%d", channel);   
    sprintf(cmd,loadedDevName, channelStr);
    if (system(cmd)!=0) {
    	ERROR1("Failed to execute MacroTune Command: %s", cmd);
    } else {
    	INFO1("Executed MacroTune Command without Error: %s", cmd);
    }
}

